#!/usr/bin/perl

use Getopt::Long;
use lib "/usr/local/nagios/libexec";
use utils qw(%ERRORS &print_revision &support &usage);

$PROGNAME="check_em01_group";

Getopt::Long::Configure('bundling');
GetOptions
  ("V"   => \$opt_V, "version"    => \$opt_V,
   "h"   => \$opt_h, "help"       => \$opt_h,
   "l=s"   => \$opt_l, "hostlist=s" => \$opt_l,
   "s=s"   => \$opt_s, "service=s" => \$opt_s,
   "f=s"   => \$opt_f, "logfile=s" => \$opt_f);

if ($opt_V) {
  print_revision($PROGNAME,'$Revision: 1.0 $');
  exit $ERRORS{'OK'};
}

if ($opt_h) {
  print_help();
  exit $ERRORS{'OK'};
}

($opt_l) || usage("Host list not specified\n");
my @hostlist = split(/\,/,$opt_l);

($opt_s) || usage("Service name not specified\n");
my $svcname = $opt_s;

($opt_f) || usage("Log file not specified\n");
my $logfile = $opt_f;

sub print_usage () {
  print "Usage: check_em01 -l <host1,host2> -s data -f /usr/local/nagios/var/service.log\n";
}

sub print_help () {
  print_revision($PROGNAME,'$Revision: 1.0 $');
  print "\nCopyright (c) 2003 Esensors, Inc.\n";
  print "\n";
  print "This plugin collects information for multiple EM01 units \n";
  print "and reports a short summary with the severity of the most\n";
  print "critical problem reported. UNKNOWN is treated as WARNING.\n";
  print "\n";
  print_usage();
  print "\n";
  print "-l, --hostlist=em01-a,em01-b,em01-c\n";
  print "   List of short host names of the em01s\n";
  print "-s, --service=data";
  print "   Service name where the em01 data is kept, which must be the same on all hosts.\n";
  print "-f, --logfile=/usr/local/nagios/var/status.log";
  print "   Location of the log file to check for individual service statuses.\n";
  print "\n";
  support();
  print "\n";
}

open LOGFILE, "<$logfile" || die "Could not open log file.";
while (<LOGFILE>) {
  chop;
  if (/\[\d*\] HOST;([^;]*);([^;]*);/) {
    $hf = $1; $hs = $2;
    foreach $host (@hostlist) {
      if (lc($host) eq lc($hf)) {
        $hstat{$host} = $hs;
      }
    }
  } elsif (/\[\d*\] SERVICE;([^;]*);([^;]*);([^;]*);.*;([^;]*)$/) {
    $hf = $1; $sf = $2; $ss = $3; $sd=$4;
    if (lc($sf) eq lc($svcname)) {
      foreach $host (@hostlist) {
        if (lc($host) eq lc($hf)) {
          $sstat{$host} = $ss;
          $sdata{$host} = $sd;
        }
      }
    }
  }
}
close LOGFILE;

$worststatus = 0;
$outstr = "";

foreach $host (@hostlist) {
  $hs = $hstat{$host};
  if ($hs eq "") {
    $hs = "UNKNOWN";
  }
  $ss = $sstat{$host};
  if ($ss eq "") {
    $ss = "UNKNOWN";
  }
  $sd = $sdata{$host};
  if ($sd =~ /temp: (.*), humidity: (.*), illumination: (.*)$/) {
    $stemp = $1;
    $shumid = $2;
  } else {
    $stemp = "UNK";
    $shumid = "UNK";
  }

  if ((uc($hs) eq "DOWN") || (uc($ss) eq "CRITICAL")) {
    $curstatus = 2;
  } elsif ((uc($ss) eq "OK") || (uc($ss) eq "RECOVERY")) {
    $curstatus = 0;
  } else {
    $curstatus = 1;
  }
  if ($curstatus > $worststatus) {
    $worststatus = $curstatus;
  }
  if ($outstr ne "") {
    $outstr .= ", ";
  }
  if (uc($hs) eq "DOWN") {
    $outstr .= "$host DOWN";
  } elsif (uc($ss) eq "UNKNOWN") {
    $outstr .= "$host UNKNOWN";
  } else {
    $outstr .= "$host $ss ($stemp/$shumid)";
  }
}

if ($worststatus >= 2) {
  print "CRITICAL: $outstr\n";
  exit $ERRORS{'CRITICAL'};
} elsif ($worststatus >= 1) {
  print "WARNING: $outstr\n";
  exit $ERRORS{'WARNING'};
} else {
  print "OK: $outstr\n";
  exit $ERRORS{'OK'};
}




